"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const graphql_1 = require("graphql");
exports.parseOperationFieldsFromAST = (operation) => {
    return operation.selectionSet.selections.reduce(
    // first level is field result name (aka query name)
    (acc, selection) => {
        if (selection.kind === 'Field') {
            selection.selectionSet.selections.forEach(
            // second level is for get names of fields asked
            (topLevel) => {
                if (topLevel.kind === 'Field') {
                    acc.push(topLevel.name.value);
                    // third level will be for sub-fields for relation (or list)
                }
            });
        }
        return acc;
    }, []);
};
exports.extractOperationsName = (schema) => {
    return Object.keys(schema.getTypeMap()).reduce((acc, type) => {
        if (type === 'Query' || type === 'Mutation') {
            const typeDef = schema.getType(type);
            if (typeDef.astNode.kind === graphql_1.Kind.OBJECT_TYPE_DEFINITION) {
                typeDef.astNode.fields.forEach((field) => {
                    acc[type][field.name.value] = graphql_1.print(field.type);
                });
            }
        }
        return acc;
    }, { Query: {}, Mutation: {} });
};
// https://stackoverflow.com/a/43849204
exports.setObjectPath = (object, path, value) => path
    .split('.')
    .reduce((o, p) => {
    return o[p] = path.split('.').pop() === p
        ? value
        : o[p] || {};
}, object);
//# sourceMappingURL=helpers.js.map