"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const graphql_shield_1 = require("graphql-shield");
const helpers_1 = require("./helpers");
const errors_1 = require("./errors");
exports.defaultAuthenticatedRule = ({ cache } = { cache: 'contextual' }) => graphql_shield_1.rule({
    cache,
})((_, args, ctx) => __awaiter(this, void 0, void 0, function* () {
    const user = ctx.user || ctx.req.user;
    if (!user || !user.id) {
        throw new errors_1.AuthenticationError();
    }
    return true;
}));
const defaultRule = (
// { cache }: { cache: ICache } = { cache: 'strict' },
{ query, fields, cache, action } = { cache: 'strict' }) => graphql_shield_1.rule({
    cache,
})((parent, args, ctx, info) => __awaiter(this, void 0, void 0, function* () {
    // Allow rule if not specific condition are provided
    if (!query && (!fields || !fields.length)) {
        return true;
    }
    // Check if specific fields are to protect
    if (fields && fields.length) {
        // TODO: check the action to know if fields to verify
        // TODO: are in the input or output
        // TODO: wait for fragment if Model need to be examined!
        // This solution allow to parse all field existing
        // But it can generate conflict between fields from differents types
        // which have the same name
        // const selection = ['Query', 'Mutation'].includes(info.parentType.name)
        //   ? print(info.fieldNodes[0].selectionSet)
        //   : parent && Object.keys(parent).join(' ')
        // //   : print(info.fieldNodes[0])
        // const result = fields.some(
        //   field => selection.indexOf(field) >= 0,
        // )
        // Get and parse top level fields requested
        const fieldsCalled = helpers_1.parseOperationFieldsFromAST(info.operation);
        const requestResult = fields.some(field => fieldsCalled.includes(field));
        if (!requestResult) {
            return true;
        }
    }
    // If fields are checked or not provided
    // Approve the user's authorization
    if (query) {
        const returnType = (['Query', 'Mutation'].includes(info.parentType.name)
            ? info.returnType
            : info.parentType).toString().replace(/!/g, '');
        const { id: requestId } = parent
            ? parent.id
                ? parent
                : { id: null }
            : args && args.where
                ? args.where
                : { id: null };
        const { id: userId = null } = ctx.user || ctx.req.user;
        const isAllowed = returnType === 'User'
            ? requestId === userId
            : yield query(ctx, requestId);
        if (isAllowed) {
            return true;
        }
    }
    throw new errors_1.AuthorizationError();
}));
exports.wrapRules = (authenticated, authenticatedRule, args) => {
    if (args.query || (args.fields && args.fields.length)) {
        return authenticated
            ? graphql_shield_1.and(authenticatedRule, defaultRule(args))
            : defaultRule(args);
    }
    return authenticatedRule;
};
//# sourceMappingURL=rules.js.map